#!/usr/bin/env python3
# -*- mode: Python; indent-space-mode: nil; coding: utf-8 -*-
import os
import pathlib
import re
import shutil
import subprocess

import click
import rpm as rpmpkg


def __run_command(args, cwd=None):
  cp = subprocess.run(args, cwd=cwd)
  cp.check_returncode()


@click.group()
@click.pass_context
def cli(ctx):
  ctx.ensure_object(dict)


@click.command()
@click.argument('spec', required=True, type=click.Path(exists=True))
@click.option('--skip-refresh', is_flag=True)
@click.pass_context
def rpm(ctx, spec, skip_refresh):
  __build(ctx, spec, skip_refresh, 'bb')


@click.command()
@click.argument('spec', required=True, type=click.Path(exists=True))
@click.option('--skip-refresh', is_flag=True)
@click.pass_context
def srpm(ctx, spec, skip_refresh):
  __build(ctx, spec, skip_refresh, 'bs')


def __build(ctx, spec, skip_refresh, type_arg):
  spec_path = os.path.abspath(spec)
  spec = rpmpkg.spec(spec_path)

  home_dir = str(pathlib.Path.home())
  spec_file_target = str(os.path.join(home_dir, 'rpmbuild/SPECS', os.path.basename(spec_path)))
  sources_dir = str(os.path.join(home_dir, 'rpmbuild/SOURCES'))
  spec_dir = str(os.path.dirname(spec_path))

  __run_command(['rpmdev-setuptree'])

  for source in spec.sources:
    source_file = source[0]
    if re.search('^http(s)?://', source_file):
      continue
    source_file_source = str(os.path.join(spec_dir, source_file))
    source_file_target = str(os.path.join(sources_dir, source_file))
    print(f"copying {source_file_source} => {source_file_target}", flush=True)
    if not os.path.exists(source_file_source) and os.path.exists(source_file_target):
      print(f"copying skipped, using existing target file. {source_file_target}", flush=True)
      continue
    shutil.copy(source_file_source, source_file_target)

  __run_command(
    ['spectool', '--get-files', '--sourcedir', spec_path],
    spec_dir
  )

  install_deps_cmd = ['dnf', 'builddep', '--assumeyes'] + ([] if skip_refresh else ['--refresh']) + ['--spec',
                                                                                                     spec_path]
  if os.getuid() != 0:
    install_deps_cmd.insert(0, 'sudo')
  __run_command(install_deps_cmd, spec_dir)

  print(f"copying {spec_path} => {spec_file_target}", flush=True)
  shutil.copy(spec_path, spec_file_target)

  __run_command(['rpmbuild', f'-{type_arg}', spec_file_target])


def main():
  cli.add_command(rpm)
  cli.add_command(srpm)
  cli()


if __name__ == "__main__":
  main()
